@@ -119,6 +119,13 @@ def get_version_data(session, url, authenticated=None):
         except KeyError:
             pass
 
+        # Otherwise if we query an endpoint like /v2.0 then we will get back
+        # just the one available version.
+        try:
+            return [body_resp['version']]
+        except KeyError:
+            pass
+
         # Older Ironic does not actually return a discovery document for the
         # single version discovery endpoint, which confuses the single-version
         # fallback logic. While there are no known other services returning
@@ -141,13 +148,6 @@ def get_version_data(session, url, authenticated=None):
                     body_resp.setdefault('version', resp.headers[header])
             return [body_resp]
 
-        # Otherwise if we query an endpoint like /v2.0 then we will get back
-        # just the one available version.
-        try:
-            return [body_resp['version']]
-        except KeyError:
-            pass
-
     err_text = resp.text[:50] + '...' if len(resp.text) > 50 else resp.text
     raise exceptions.DiscoveryFailure('Invalid Response - Bad version data '
                                       'returned: %s' % err_text)
@@ -473,6 +473,12 @@ def _version_from_url(url):
     url = urllib.parse.urlparse(url)
     for part in reversed(url.path.split('/')):
         try:
+            # All integer project ids can parse as valid versions. In URLs
+            # all known instances of versions start with a v. So check to make
+            # sure the url part starts with 'v', then check that it parses
+            # as a valid version.
+            if part[0] != 'v':
+                continue
             return normalize_version_number(part)
         except Exception:
             pass
@@ -1415,6 +1421,18 @@ def get_discovery(session, url, cache=None, authenticated=False):
     if session.auth and hasattr(session.auth, '_discovery_cache'):
         caches.append(session.auth._discovery_cache)
 
+    # https://example.com and https://example.com/ should be treated the same
+    # for caching purposes.
+    parsed_url = urllib.parse.urlparse(url)
+    if parsed_url.path in ('', '/'):
+        url = urllib.parse.ParseResult(
+            parsed_url.scheme,
+            parsed_url.netloc,
+            '',
+            parsed_url.params,
+            parsed_url.query,
+            parsed_url.fragment).geturl()
+
     for cache in caches:
         disc = cache.get(url)
 
@@ -1425,7 +1443,7 @@ def get_discovery(session, url, cache=None, authenticated=False):
 
     # Whether we get one from fetching or from cache, set it in the
     # caches. This assures that if we combine sessions and auth plugins
-    # that we don't make unnecesary calls.
+    # that we don't make unnecessary calls.
     if disc:
         for cache in caches:
             cache[url] = disc
